import { Meta, StoryObj } from '@storybook/angular';

import iconsetIcons from 'src/assets/iconset-icons.json';
import ionicIcons from 'src/assets/ionic-icons.json';

/**
 * Dynamically load all available custom assets (SVGs and PNGs) from the `src/assets/img` folder.
 */
const reqSvgsPngs = require.context('./img', true, /\.(svg|png)$/);
const customAssets = Array.from(new Set(reqSvgsPngs.keys().map(fixCustomIconsPath))).map((iconPath) => {
  const [iconName] = iconPath.split('/').pop()!.split('.');

  return {
    name: iconName,
    path: iconPath,
  };
});

/**
 * The assets path at the same level as require.context root, will have relative paths, which start with `./`,
 * while the child paths will have absolute paths, which start with `src/assets/img`.
 * This function makes all paths to start from `/assets/img/`.
 * @param iconPath
 */
function fixCustomIconsPath(iconPath: string): string {
  if (iconPath.includes('src/assets')) {
    return iconPath.replace('src', '');
  } else if (iconPath.startsWith('./')) {
    return iconPath.replace('./', '/assets/img/');
  }

  return iconPath;
}

/**
 * @link https://ionic.io/ionicons
 */
export default {
  title: 'Icons',
  parameters: {
    viewport: {
      defaultViewport: 'reset',
    },
  },
  args: {
    filter: '',
  },
} as Meta;

type IconType = {
  name: string;
  tags: string[];
};

function renderTemplate(templateString: string, amountOfIcons: number): string {
  return `
    <style>
      header {
        display: flex;
        justify-content: flex-end;
        padding: 8px;
      }
      .icons {
        display: grid;
        grid-template-columns: repeat(auto-fill, minmax(100px, 1fr));
        grid-gap: 4px;
        padding: 8px;
      }

      .icon {
        background-color: #f6f9fc;
        display: flex;
        flex-direction: column;
        align-items: center;
        justify-content: center;
        border: 1px solid #d9e8f2;
        border-radius: 4px;
        transition: all 200ms ease-out;
        padding: 8px;
        min-height: 80px;
        box-shadow: 0 0 0 0 rgba(0, 0, 0, 0.4);
      }

      .icon:hover {
        transform: scale(1.15);
        box-shadow: 0 5px 10px 0 rgba(0, 0, 0, 0.4);
      }

      .icon-image {
        flex-grow: 1;
        display: flex;
        align-items: center;
        justify-content: center;
        min-width: 30px;
      }

      .icon-label {
        font-size: 12px;
        margin-top: 4px;
        text-align: center;
        word-break: break-word;
      }

      img {
        max-height: 100px;
        max-width: 100%;
        min-width: 0;
      }
    </style>

    <header>${amountOfIcons} icons visible</header>

    <div class="icons">
      ${templateString}
    </div>
  `;
}

export const ThirdParty: StoryObj = {
  args: {
    icon: 'filled',
    library: 'all',
  },
  argTypes: {
    icon: {
      options: ['outline', 'filled', 'sharp'],
      control: {
        type: 'select',
      },
    },
    library: {
      options: ['ionic', 'iconset', 'all'],
      control: {
        type: 'select',
      },
    },
  },
  render(args) {
    let icons: IconType[] = [];

    switch (args.library) {
      case 'ionic':
        icons = ionicIcons;
        break;
      case 'iconset':
        icons = iconsetIcons.map((name) => ({ name, tags: [] }));
        break;
      default:
        icons = [...ionicIcons, ...iconsetIcons.map((name) => ({ name, tags: [] }))];
        break;
    }

    const visibleIcons = icons
      .filter((icon) => {
        if (args.icon === 'filled') {
          return !icon.name.includes('-outline') && !icon.name.includes('-sharp');
        } else {
          return icon.name.includes(`-${args.icon}`);
        }
      })
      .filter(
        (icon) =>
          args.filter === '' ||
          [icon.name, ...icon.tags].some((tag) => tag.toLowerCase().includes(args.filter.toLowerCase()))
      )
      .sort((a, b) => a.name.localeCompare(b.name));

    return {
      template: renderTemplate(
        visibleIcons
          .map(
            (icon) => `
              <div class="icon">
                <div class="icon-image">
                  ${
                    icon.name.includes('cicon')
                      ? `<div class="${icon.name}"></div>`
                      : `<ion-icon name="${icon.name}" size="large"></ion-icon>`
                  }
                </div>
                <label class="icon-label">${icon.name}</label>
              </div>
            `
          )
          .join(''),
        visibleIcons.length
      ),
    };
  },
};

export const CustomAssets: StoryObj = {
  render(args) {
    const visibleIcons = customAssets
      .filter((icon) => args.filter === '' || icon.name.toLowerCase().includes(args.filter.toLowerCase()))
      .sort((a, b) => a.name.localeCompare(b.name));

    return {
      template: renderTemplate(
        visibleIcons
          .map((icon) => {
            return `
              <button class="icon" title="${icon.name}" popovertarget="${icon.path}">
                <div class="icon-image">
                  <img src="${icon.path}" alt="${icon.name}" />
                </div>
                <label class="icon-label">${icon.name}</label>
                <div popover id="${icon.path}" class="icon-popover">${icon.path}</div>
              </button>
            `;
          })
          .join(' '),
        visibleIcons.length
      ),
    };
  },
};
